@@ -2,18 +2,17 @@ import io
 from typing import List, overload, Sequence, TYPE_CHECKING, Union
 
 import tiledb.cc as lt
-from .ctx import default_ctx
+from .ctx import CtxMixin
 
 if TYPE_CHECKING:
     from .libtiledb import Ctx
 
 
-class Filter(lt.Filter):
+class Filter(CtxMixin, lt.Filter):
     """Base class for all TileDB filters."""
 
     def __init__(self, type: lt.FilterOption, ctx: "Ctx" = None):
-        self._ctx = ctx or default_ctx()
-        super().__init__(self._ctx, type)
+        super().__init__(ctx, type)
 
     def __repr__(self) -> str:
         output = io.StringIO()
@@ -78,14 +77,9 @@ class CompressionFilter(Filter):
     def __init__(self, type: lt.FilterType, level: int = -1, ctx: "Ctx" = None):
         if not isinstance(level, int):
             raise ValueError("`level` argument must be a int")
-        self._ctx = ctx or default_ctx()
 
-        super().__init__(type, self._ctx)
-        self._set_option(
-            self._ctx,
-            lt.FilterOption.COMPRESSION_LEVEL,
-            level,
-        )
+        super().__init__(type, ctx)
+        self._set_option(self._ctx, lt.FilterOption.COMPRESSION_LEVEL, level)
 
     @property
     def level(self):
@@ -96,9 +90,7 @@ class NoOpFilter(Filter):
     """A filter that does nothing."""
 
     def __init__(self, ctx: "Ctx" = None):
-        self._ctx = ctx or default_ctx()
-
-        super().__init__(lt.FilterType.NONE, self._ctx)
+        super().__init__(lt.FilterType.NONE, ctx)
 
     def _attrs_(self):
         return {}
@@ -129,9 +121,7 @@ class GzipFilter(CompressionFilter):
         if not isinstance(level, int):
             raise ValueError("`level` argument must be a int")
 
-        self._ctx = ctx or default_ctx()
-
-        super().__init__(lt.FilterType.GZIP, level, self._ctx)
+        super().__init__(lt.FilterType.GZIP, level, ctx)
 
     def _attrs_(self):
         return {"level": self.level}
@@ -162,9 +152,7 @@ class ZstdFilter(CompressionFilter):
         if not isinstance(level, int):
             raise ValueError("`level` argument must be a int")
 
-        self._ctx = ctx or default_ctx()
-
-        super().__init__(lt.FilterType.ZSTD, level, self._ctx)
+        super().__init__(lt.FilterType.ZSTD, level, ctx)
 
     def _attrs_(self):
         return {"level": self.level}
@@ -195,9 +183,7 @@ class LZ4Filter(CompressionFilter):
         if not isinstance(level, int):
             raise ValueError("`level` argument must be a int")
 
-        self._ctx = ctx or default_ctx()
-
-        super().__init__(lt.FilterType.LZ4, level, self._ctx)
+        super().__init__(lt.FilterType.LZ4, level, ctx)
 
     def _attrs_(self):
         return {"level": self.level}
@@ -226,9 +212,7 @@ class Bzip2Filter(CompressionFilter):
         if not isinstance(level, int):
             raise ValueError("`level` argument must be a int")
 
-        self._ctx = ctx or default_ctx()
-
-        super().__init__(lt.FilterType.BZIP2, level, self._ctx)
+        super().__init__(lt.FilterType.BZIP2, level, ctx)
 
     def _attrs_(self):
         return {"level": self.level}
@@ -257,9 +241,7 @@ class RleFilter(CompressionFilter):
         if not isinstance(level, int):
             raise ValueError("`level` argument must be a int")
 
-        self._ctx = ctx or default_ctx()
-
-        super().__init__(lt.FilterType.RLE, level, self._ctx)
+        super().__init__(lt.FilterType.RLE, level, ctx)
 
     def _attrs_(self):
         return {}
@@ -288,9 +270,7 @@ class DoubleDeltaFilter(CompressionFilter):
         if not isinstance(level, int):
             raise ValueError("`level` argument must be a int")
 
-        self._ctx = ctx or default_ctx()
-
-        super().__init__(lt.FilterType.DOUBLE_DELTA, level, self._ctx)
+        super().__init__(lt.FilterType.DOUBLE_DELTA, level, ctx)
 
     def _attrs_(self):
         return {}
@@ -319,9 +299,7 @@ class DictionaryFilter(CompressionFilter):
         if not isinstance(level, int):
             raise ValueError("`level` argument must be a int")
 
-        self._ctx = ctx or default_ctx()
-
-        super().__init__(lt.FilterType.DICTIONARY, level, self._ctx)
+        super().__init__(lt.FilterType.DICTIONARY, level, ctx)
 
     def _attrs_(self):
         return {}
@@ -344,8 +322,7 @@ class BitShuffleFilter(Filter):
     """
 
     def __init__(self, ctx: "Ctx" = None):
-        self._ctx = ctx or default_ctx()
-        super().__init__(lt.FilterType.BITSHUFFLE, self._ctx)
+        super().__init__(lt.FilterType.BITSHUFFLE, ctx)
 
     def _attrs_(self):
         return {}
@@ -368,8 +345,7 @@ class ByteShuffleFilter(Filter):
     """
 
     def __init__(self, ctx: "Ctx" = None):
-        self._ctx = ctx or default_ctx()
-        super().__init__(lt.FilterType.BYTESHUFFLE, self._ctx)
+        super().__init__(lt.FilterType.BYTESHUFFLE, ctx)
 
     def _attrs_(self):
         return {}
@@ -398,16 +374,11 @@ class BitWidthReductionFilter(Filter):
     def __init__(self, window: int = -1, ctx: "Ctx" = None):
         if not isinstance(window, int):
             raise ValueError("`window` argument must be a int")
-        self._ctx = ctx or default_ctx()
 
-        super().__init__(lt.FilterType.BIT_WIDTH_REDUCTION)
+        super().__init__(lt.FilterType.BIT_WIDTH_REDUCTION, ctx)
 
         if window != -1:
-            self._set_option(
-                self._ctx,
-                lt.FilterOption.BIT_WIDTH_MAX_WINDOW,
-                window,
-            )
+            self._set_option(self._ctx, lt.FilterOption.BIT_WIDTH_MAX_WINDOW, window)
 
     def _attrs_(self):
         return {"window": self.window}
@@ -440,14 +411,11 @@ class PositiveDeltaFilter(Filter):
     def __init__(self, window: int = -1, ctx: "Ctx" = None):
         if not isinstance(window, int):
             raise ValueError("`window` argument must be a int")
-        self._ctx = ctx or default_ctx()
 
-        super().__init__(lt.FilterType.POSITIVE_DELTA)
+        super().__init__(lt.FilterType.POSITIVE_DELTA, ctx)
         if window != -1:
             self._set_option(
-                self._ctx,
-                lt.FilterOption.POSITIVE_DELTA_MAX_WINDOW,
-                window,
+                self._ctx, lt.FilterOption.POSITIVE_DELTA_MAX_WINDOW, window
             )
 
     def _attrs_(self):
@@ -475,8 +443,7 @@ class ChecksumMD5Filter(Filter):
     """
 
     def __init__(self, ctx: "Ctx" = None):
-        self._ctx = ctx or default_ctx()
-        super().__init__(lt.FilterType.CHECKSUM_MD5, self._ctx)
+        super().__init__(lt.FilterType.CHECKSUM_MD5, ctx)
 
     def _attrs_(self):
         return {}
@@ -499,8 +466,7 @@ class ChecksumSHA256Filter(Filter):
     """
 
     def __init__(self, ctx: "Ctx" = None):
-        self._ctx = ctx or default_ctx()
-        super().__init__(lt.FilterType.CHECKSUM_SHA256, self._ctx)
+        super().__init__(lt.FilterType.CHECKSUM_SHA256, ctx)
 
     def _attrs_(self):
         return {}
@@ -544,37 +510,27 @@ class FloatScaleFilter(Filter):
         self._factor = factor
         self._offset = offset
         self._bytewidth = bytewidth
-        self._ctx = ctx or default_ctx()
-
-        super().__init__(lt.FilterType.SCALE_FLOAT, self._ctx)
+        super().__init__(lt.FilterType.SCALE_FLOAT, ctx)
 
         if factor:
             self._set_option(
-                self._ctx,
-                lt.FilterOption.SCALE_FLOAT_FACTOR,
-                float(factor),
+                self._ctx, lt.FilterOption.SCALE_FLOAT_FACTOR, float(factor)
             )
 
         if offset is not None:
             # 0 evals to false so we weren't actually setting here
             # if offset==0 :upside-down-face:
             self._set_option(
-                self._ctx,
-                lt.FilterOption.SCALE_FLOAT_OFFSET,
-                float(offset),
+                self._ctx, lt.FilterOption.SCALE_FLOAT_OFFSET, float(offset)
             )
 
         if bytewidth:
             self._set_option(
-                self._ctx,
-                lt.FilterOption.SCALE_FLOAT_BYTEWIDTH,
-                bytewidth,
+                self._ctx, lt.FilterOption.SCALE_FLOAT_BYTEWIDTH, bytewidth
             )
 
     def dump(self):
-        self._dump(
-            self._ctx,
-        )
+        self._dump(self._ctx)
 
     def _attrs_(self):
         return {
@@ -613,8 +569,7 @@ class XORFilter(Filter):
     """
 
     def __init__(self, ctx: "Ctx" = None):
-        self._ctx = ctx or default_ctx()
-        super().__init__(lt.FilterType.XOR, self._ctx)
+        super().__init__(lt.FilterType.XOR, ctx)
 
     def _attrs_(self):
         return {}
@@ -642,9 +597,11 @@ class WebpFilter(Filter):
     On read, this filter decodes WebP data and returns raw colorspace values to
     the caller.
 
-    This filter expects the array to provide two dimensions for Y, X pixel
-    position. Dimensions may be defined with any name, but Y, X should be at
-    dimension index 0, 1 respectively.
+    This filter expects the array to provide two dimensions for Y, X pixel position.
+    Dimensions may be defined with any name, but Y, X should be at dimension index 0, 1 respectively.
+    Dimensions can be any two matching integral types, such as `{uint64_t, uint64_t}` or `{int64_t, int64_t}`.
+
+    The WebP filter supports only the `uint8_t` type for attributes.
 
     :param quality: quality_factor setting for lossy WebP compression
     :type quality: float in range [0.0, 100.0]
@@ -687,9 +644,7 @@ class WebpFilter(Filter):
         self._input_format = input_format
         self._quality = quality
         self._lossless = lossless
-        self._ctx = ctx or default_ctx()
-
-        super().__init__(lt.FilterType.WEBP, self._ctx)
+        super().__init__(lt.FilterType.WEBP, ctx)
 
         if input_format is not None:
             self._set_option(
@@ -699,18 +654,10 @@ class WebpFilter(Filter):
             )
 
         if quality is not None:
-            self._set_option(
-                self._ctx,
-                lt.FilterOption.WEBP_QUALITY,
-                float(quality),
-            )
+            self._set_option(self._ctx, lt.FilterOption.WEBP_QUALITY, float(quality))
 
         if lossless is not None:
-            self._set_option(
-                self._ctx,
-                lt.FilterOption.WEBP_LOSSLESS,
-                lossless,
-            )
+            self._set_option(self._ctx, lt.FilterOption.WEBP_LOSSLESS, lossless)
 
     def _attrs_(self):
         return {
@@ -722,29 +669,20 @@ class WebpFilter(Filter):
     @property
     def input_format(self):
         return lt.WebpInputFormat(
-            self._get_option(
-                self._ctx,
-                lt.FilterOption.WEBP_INPUT_FORMAT,
-            )
+            self._get_option(self._ctx, lt.FilterOption.WEBP_INPUT_FORMAT)
         )
 
     @property
     def quality(self):
-        return self._get_option(
-            self._ctx,
-            lt.FilterOption.WEBP_QUALITY,
-        )
+        return self._get_option(self._ctx, lt.FilterOption.WEBP_QUALITY)
 
     @property
     def lossless(self):
-        return self._get_option(
-            self._ctx,
-            lt.FilterOption.WEBP_LOSSLESS,
-        )
+        return self._get_option(self._ctx, lt.FilterOption.WEBP_LOSSLESS)
 
 
 #
-class FilterList(lt.FilterList):
+class FilterList(CtxMixin, lt.FilterList):
     """
     An ordered list of Filter objects for filtering TileDB data.
 
@@ -796,30 +734,16 @@ class FilterList(lt.FilterList):
     }
 
     def __init__(
-        self,
-        filters: Sequence[Filter] = None,
-        chunksize: int = None,
-        ctx: "Ctx" = None,
-        _lt_obj: lt.FilterList = None,
-        _capsule: "PyCapsule" = None,
+        self, filters: Sequence[Filter] = None, chunksize: int = None, ctx: "Ctx" = None
     ):
-        self._ctx = ctx or default_ctx()
-
-        if _capsule is not None:
-            super().__init__(self._ctx, _capsule)
-        elif _lt_obj is not None:
-            super().__init__(_lt_obj)
-        else:
-            super().__init__(self._ctx)
-            if filters is not None:
-                filters = list(filters)
-                for f in filters:
-                    if not isinstance(f, Filter):
-                        raise ValueError(
-                            "filters argument must be an iterable of TileDB filter objects"
-                        )
-                    self._add_filter(f)
-
+        super().__init__(ctx)
+        if filters is not None:
+            for f in filters:
+                if not isinstance(f, Filter):
+                    raise ValueError(
+                        "filters argument must be an iterable of TileDB filter objects"
+                    )
+                self._add_filter(f)
         if chunksize is not None:
             self._chunksize = chunksize
 
@@ -903,9 +827,7 @@ class FilterList(lt.FilterList):
         self._add_filter(filter)
 
     def __repr__(self) -> str:
-        filters = ",\n       ".join(
-            [repr(self._getfilter(i)) for i in range(len(self))]
-        )
+        filters = ",\n       ".join(repr(self._getfilter(i)) for i in range(len(self)))
         return "FilterList([{0!s}])".format(filters)
 
     def _repr_html_(self) -> str:
@@ -952,12 +874,5 @@ class FilterList(lt.FilterList):
         else:
             options = []
 
-        _cctx = self._ctx
-
-        opts = []
-        for opt in options:
-            opts.append(fil._get_option(_cctx, opt))
-
-        fil = filtype(*opts, ctx=self._ctx)
-
-        return fil
+        ctx = self._ctx
+        return filtype(*(fil._get_option(ctx, opt) for opt in options), ctx=ctx)
